@@ -72,11 +72,46 @@ public:
     void keygen(const byte* IKM, size_t IKM_len,
                 const std::string& info = "")
     {   blst_keygen(&key, IKM, IKM_len, C_bytes(info.data()), info.size());   }
+    void keygen_v3(const byte* IKM, size_t IKM_len,
+                   const std::string& info = "")
+    {   blst_keygen_v3(&key, IKM, IKM_len, C_bytes(info.data()), info.size());   }
+    void keygen_v4_5(const byte* IKM, size_t IKM_len,
+                     const byte* salt, size_t salt_len,
+                     const std::string& info = "")
+    {   blst_keygen_v4_5(&key, IKM, IKM_len, salt, salt_len,
+                               C_bytes(info.data()), info.size());
+    }
+    void keygen_v5(const byte* IKM, size_t IKM_len,
+                   const byte* salt, size_t salt_len,
+                   const std::string& info = "")
+    {   blst_keygen_v5(&key, IKM, IKM_len, salt, salt_len,
+                             C_bytes(info.data()), info.size());
+    }
 #if __cplusplus >= 201703L
     void keygen(const app__string_view IKM, // string_view by value, cool!
                 const std::string& info = "")
     {   keygen(C_bytes(IKM.data()), IKM.size(), info);   }
+    void keygen_v3(const app__string_view IKM, // string_view by value, cool!
+                   const std::string& info = "")
+    {   keygen_v3(C_bytes(IKM.data()), IKM.size(), info);   }
+    void keygen_v4_5(const app__string_view IKM, // string_view by value, cool!
+                     const app__string_view salt,
+                     const std::string& info = "")
+    {   keygen_v4_5(C_bytes(IKM.data()), IKM.size(),
+                    C_bytes(salt.data()), salt.size(), info);
+    }
+    void keygen_v5(const app__string_view IKM, // string_view by value, cool!
+                   const app__string_view salt,
+                   const std::string& info = "")
+    {   keygen_v5(C_bytes(IKM.data()), IKM.size(),
+                  C_bytes(salt.data()), salt.size(), info);
+    }
 #endif
+    void derive_master_eip2333(const byte* IKM, size_t IKM_len)
+    {   blst_derive_master_eip2333(&key, IKM, IKM_len);   }
+    void derive_child_eip2333(const SecretKey& SK, unsigned int child_index)
+    {   blst_derive_child_eip2333(&key, &SK.key, child_index);   }
+
     void from_bendian(const byte in[32]) { blst_scalar_from_bendian(&key, in); }
     void from_lendian(const byte in[32]) { blst_scalar_from_lendian(&key, in); }
 
@@ -86,7 +121,7 @@ public:
     {   blst_lendian_from_scalar(out, &key);   }
 };
 
-struct Scalar {
+class Scalar {
 private:
     blst_scalar val;
 
@@ -106,13 +141,25 @@ public:
     {   blst_lendian_from_scalar(out, &val);   }
 
     Scalar* add(const Scalar& a)
-    {   return blst_sk_add_n_check(&val, &val, a) ? this : nullptr;   }
+    {   if (!blst_sk_add_n_check(&val, &val, a))
+            throw BLST_BAD_SCALAR;
+        return this;
+    }
     Scalar* add(const SecretKey& a)
-    {   return blst_sk_add_n_check(&val, &val, &a.key) ? this : nullptr;   }
+    {   if (!blst_sk_add_n_check(&val, &val, &a.key))
+            throw BLST_BAD_SCALAR;
+        return this;
+    }
     Scalar* sub(const Scalar& a)
-    {   return blst_sk_sub_n_check(&val, &val, a) ? this : nullptr;   }
+    {   if (!blst_sk_sub_n_check(&val, &val, a))
+            throw BLST_BAD_SCALAR;
+        return this;
+    }
     Scalar* mul(const Scalar& a)
-    {   return blst_sk_mul_n_check(&val, &val, a) ? this : nullptr;   }
+    {   if (!blst_sk_mul_n_check(&val, &val, a))
+            throw BLST_BAD_SCALAR;
+        return this;
+    }
     Scalar* inverse()
     {   blst_sk_inverse(&val, &val); return this;   }
 
@@ -130,11 +177,20 @@ private:
     P1_Affine(const blst_p1_affine *cptr) { point = *cptr; }
 public:
     P1_Affine() { memset(&point, 0, sizeof(point)); }
+#ifndef SWIG
     P1_Affine(const byte *in)
     {   BLST_ERROR err = blst_p1_deserialize(&point, in);
         if (err != BLST_SUCCESS)
             throw err;
     }
+#endif
+    P1_Affine(const byte *in, size_t len)
+    {   if (len == 0 || len != (in[0]&0x80 ? 48 : 96))
+            throw BLST_BAD_ENCODING;
+        BLST_ERROR err = blst_p1_deserialize(&point, in);
+        if (err != BLST_SUCCESS)
+            throw err;
+    }
     P1_Affine(const P1& jacobian);
 
     P1_Affine dup() const { return *this; }
@@ -166,7 +222,7 @@ private:
     friend class P2_Affine;
     friend class PT;
     friend class P1;
-    friend class P1s;
+    friend class P1_Affines;
     operator const blst_p1_affine*() const { return &point; }
     operator blst_p1_affine*()             { return &point; }
 };
@@ -179,6 +235,7 @@ private:
 public:
     P1() { memset(&point, 0, sizeof(point)); }
     P1(const SecretKey& sk) { blst_sk_to_pk_in_g1(&point, &sk.key); }
+#ifndef SWIG
     P1(const byte *in)
     {   blst_p1_affine a;
         BLST_ERROR err = blst_p1_deserialize(&a, in);
@@ -186,6 +243,16 @@ public:
             throw err;
         blst_p1_from_affine(&point, &a);
     }
+#endif
+    P1(const byte *in, size_t len)
+    {   if (len == 0 || len != (in[0]&0x80 ? 48 : 96))
+            throw BLST_BAD_ENCODING;
+        blst_p1_affine a;
+        BLST_ERROR err = blst_p1_deserialize(&a, in);
+        if (err != BLST_SUCCESS)
+            throw err;
+        blst_p1_from_affine(&point, &a);
+    }
     P1(const P1_Affine& affine) { blst_p1_from_affine(&point, affine); }
 
     P1 dup() const                      { return *this; }
@@ -260,111 +327,141 @@ public:
 
 private:
     friend class P1_Affine;
-    friend class P1s;
+    friend class P1_Affines;
     operator const blst_p1*() const { return &point; }
     operator blst_p1*()             { return &point; }
 };
 
-class P1s {
+class P1_Affines {
 private:
     struct p1_affine_no_init {
         blst_p1_affine point;
         p1_affine_no_init() { }
-        operator blst_p1_affine*() { return &point; }
+        operator blst_p1_affine*()              { return &point; }
+        operator const blst_p1_affine*() const  { return &point; }
     };
 
     std::vector<p1_affine_no_init> table;
     size_t wbits, npoints;
 
 public:
-    P1s() {}
 #ifndef SWIG
-    P1s(size_t wbits, const P1_Affine* points[], size_t npoints)
+    P1_Affines() {}
+    P1_Affines(size_t wbits, const P1_Affine* const points[], size_t npoints)
     {   this->wbits = wbits;
         this->npoints = npoints;
         table.resize(npoints << (wbits-1));
         blst_p1s_mult_wbits_precompute(table[0], wbits,
-                    reinterpret_cast<const blst_p1_affine **>(points), npoints);
+                        reinterpret_cast<const blst_p1_affine *const*>(points),
+                        npoints);
     }
-    P1s(size_t wbits, const P1_Affine points[], size_t npoints)
-    {   const P1_Affine* ptrs[2] = { points, nullptr };
-        P1s(wbits, ptrs, npoints);
+    P1_Affines(size_t wbits, const P1_Affine points[], size_t npoints)
+    {   const P1_Affine* const ptrs[2] = { points, nullptr };
+        P1_Affines(wbits, ptrs, npoints);
     }
-    P1s(size_t wbits, const std::vector<P1_Affine> points)
-    {   P1s(wbits, &points[0], points.size());   }
+    P1_Affines(size_t wbits, const std::vector<P1_Affine> points)
+    {   P1_Affines(wbits, &points[0], points.size());   }
 
-    P1s(size_t wbits, const P1* points[], size_t npoints)
+    P1_Affines(size_t wbits, const P1* const points[], size_t npoints)
     {   size_t cap = npoints << (wbits-1);
 
         this->wbits = wbits;
         this->npoints = npoints;
         table.resize(cap);
         blst_p1s_to_affine(table[cap-npoints],
-                           reinterpret_cast<const blst_p1 **>(points), npoints);
-        const blst_p1_affine* ptrs[2] = { table[cap-npoints], nullptr };
+                           reinterpret_cast<const blst_p1 *const*>(points),
+                           npoints);
+        const blst_p1_affine* const ptrs[2] = { table[cap-npoints], nullptr };
         blst_p1s_mult_wbits_precompute(table[0], wbits, ptrs, npoints);
     }
-    P1s(size_t wbits, const P1 points[], size_t npoints)
-    {   const P1* ptrs[2] = { points, nullptr };
-        P1s(wbits, ptrs, npoints);
+    P1_Affines(size_t wbits, const P1 points[], size_t npoints)
+    {   const P1* const ptrs[2] = { points, nullptr };
+        P1_Affines(wbits, ptrs, npoints);
     }
-    P1s(size_t wbits, const std::vector<P1> points)
-    {   P1s(wbits, &points[0], points.size());   }
+    P1_Affines(size_t wbits, const std::vector<P1> points)
+    {   P1_Affines(wbits, &points[0], points.size());   }
 
-    P1 mult_wbits(const byte* scalars[], size_t nbits)
+    P1_Affines(const P1* const points[], size_t npoints)
+    {   this->wbits = 0;
+        this->npoints = npoints;
+        table.resize(npoints);
+        blst_p1s_to_affine(table[0],
+                           reinterpret_cast<const blst_p1 *const*>(points),
+                           npoints);
+    }
+    P1_Affines(const P1 points[], size_t npoints)
+    {   const P1* const ptrs[2] = { points, nullptr };
+        P1_Affines(ptrs, npoints);
+    }
+    P1_Affines(const std::vector<P1> points)
+    {   P1_Affines(&points[0], points.size());   }
+
+    P1 mult(const byte* const scalars[], size_t nbits) const
     {   P1 ret;
-        void* scratch = malloc(blst_p1s_mult_wbits_scratch_sizeof(npoints));
-        blst_p1s_mult_wbits(ret, table[0], wbits, npoints,
-                                 scalars, nbits, scratch);
-        free(scratch);
+        limb_t* scratch;
+
+        if (wbits != 0) {
+            scratch = new limb_t[blst_p1s_mult_wbits_scratch_sizeof(npoints)/sizeof(limb_t)];
+            blst_p1s_mult_wbits(ret, table[0], wbits, npoints,
+                                     scalars, nbits, scratch);
+        } else {
+            scratch = new limb_t[blst_p1s_mult_pippenger_scratch_sizeof(npoints)/sizeof(limb_t)];
+            const blst_p1_affine* const ptrs[2] = { table[0], nullptr };
+            blst_p1s_mult_pippenger(ret, ptrs, npoints,
+                                         scalars, nbits, scratch);
+        }
+        delete[] scratch;
         return ret;
     }
 
-    static std::vector<P1_Affine> to_affine(const P1* points[], size_t npoints)
+    static std::vector<P1_Affine> from(const P1* const points[], size_t npoints)
     {   std::vector<P1_Affine> ret;
         ret.resize(npoints);
-        blst_p1s_to_affine(ret[0], reinterpret_cast<const blst_p1 **>(points),
-                                   npoints);
+        blst_p1s_to_affine(ret[0],
+                           reinterpret_cast<const blst_p1 *const*>(points),
+                           npoints);
         return ret;
     }
-    static std::vector<P1_Affine> to_affine(const P1 points[], size_t npoints)
-    {   const P1* ptrs[2] = { points, nullptr };
-        return to_affine(ptrs, npoints);
+    static std::vector<P1_Affine> from(const P1 points[], size_t npoints)
+    {   const P1* const ptrs[2] = { points, nullptr };
+        return from(ptrs, npoints);
     }
-    static std::vector<P1_Affine> to_affine(std::vector<P1> points)
-    {   return to_affine(&points[0], points.size());   }
+    static std::vector<P1_Affine> from(std::vector<P1> points)
+    {   return from(&points[0], points.size());   }
 #endif
 
-    static P1 mult_pippenger(const P1_Affine* points[], size_t npoints,
-                             const byte* scalars[], size_t nbits)
+    static P1 mult_pippenger(const P1_Affine* const points[], size_t npoints,
+                             const byte* const scalars[], size_t nbits)
     {   P1 ret;
-        void* scratch = malloc(blst_p1s_mult_pippenger_scratch_sizeof(npoints));
+        limb_t* scratch;
+        scratch = new limb_t[blst_p1s_mult_pippenger_scratch_sizeof(npoints)/sizeof(limb_t)];
         blst_p1s_mult_pippenger(ret,
-                    reinterpret_cast<const blst_p1_affine **>(points), npoints,
-                    scalars, nbits, scratch);
-        free(scratch);
+                    reinterpret_cast<const blst_p1_affine *const*>(points),
+                    npoints, scalars, nbits, scratch);
+        delete[] scratch;
         return ret;
     }
 #ifndef SWIG
     static P1 mult_pippenger(const P1_Affine points[], size_t npoints,
-                             const byte* scalars[], size_t nbits)
-    {   const P1_Affine *ptrs[2] = { points, nullptr };
+                             const byte* const scalars[], size_t nbits)
+    {   const P1_Affine* const ptrs[2] = { points, nullptr };
         return mult_pippenger(ptrs, npoints, scalars, nbits);
     }
     static P1 mult_pippenger(const std::vector<P1_Affine> points,
-                             const byte* scalars[], size_t nbits)
+                             const byte* const scalars[], size_t nbits)
     {   return mult_pippenger(&points[0], points.size(), scalars, nbits);   }
 #endif
 
-    static P1 add(const P1_Affine* points[], size_t npoints)
+    static P1 add(const P1_Affine* const points[], size_t npoints)
     {   P1 ret;
-        blst_p1s_add(ret, reinterpret_cast<const blst_p1_affine **>(points),
-                          npoints);
+        blst_p1s_add(ret,
+                     reinterpret_cast<const blst_p1_affine *const*>(points),
+                     npoints);
         return ret;
     }
 #ifndef SWIG
     static P1 add(const P1_Affine points[], size_t npoints)
-    {   const P1_Affine *ptrs[2] = { points, nullptr };
+    {   const P1_Affine* const ptrs[2] = { points, nullptr };
         return add(ptrs, npoints);
     }
     static P1 add(const std::vector<P1_Affine> points)
@@ -379,11 +476,20 @@ private:
     P2_Affine(const blst_p2_affine *cptr) { point = *cptr; }
 public:
     P2_Affine() { memset(&point, 0, sizeof(point)); }
+#ifndef SWIG
     P2_Affine(const byte *in)
     {   BLST_ERROR err = blst_p2_deserialize(&point, in);
         if (err != BLST_SUCCESS)
             throw err;
     }
+#endif
+    P2_Affine(const byte *in, size_t len)
+    {   if (len == 0 || len != (in[0]&0x80 ? 96 : 192))
+            throw BLST_BAD_ENCODING;
+        BLST_ERROR err = blst_p2_deserialize(&point, in);
+        if (err != BLST_SUCCESS)
+            throw err;
+    }
     P2_Affine(const P2& jacobian);
 
     P2_Affine dup() const { return *this; }
@@ -415,7 +521,7 @@ private:
     friend class P1_Affine;
     friend class PT;
     friend class P2;
-    friend class P2s;
+    friend class P2_Affines;
     operator const blst_p2_affine*() const { return &point; }
     operator blst_p2_affine*()             { return &point; }
 };
@@ -428,6 +534,7 @@ private:
 public:
     P2() { memset(&point, 0, sizeof(point)); }
     P2(const SecretKey& sk) { blst_sk_to_pk_in_g2(&point, &sk.key); }
+#ifndef SWIG
     P2(const byte *in)
     {   blst_p2_affine a;
         BLST_ERROR err = blst_p2_deserialize(&a, in);
@@ -435,6 +542,16 @@ public:
             throw err;
         blst_p2_from_affine(&point, &a);
     }
+#endif
+    P2(const byte *in, size_t len)
+    {   if (len == 0 || len != (in[0]&0x80 ? 96 : 192))
+            throw BLST_BAD_ENCODING;
+        blst_p2_affine a;
+        BLST_ERROR err = blst_p2_deserialize(&a, in);
+        if (err != BLST_SUCCESS)
+            throw err;
+        blst_p2_from_affine(&point, &a);
+    }
     P2(const P2_Affine& affine) { blst_p2_from_affine(&point, affine); }
 
     P2 dup() const                      { return *this; }
@@ -509,111 +626,141 @@ public:
 
 private:
     friend class P2_Affine;
-    friend class P2s;
+    friend class P2_Affines;
     operator const blst_p2*() const { return &point; }
     operator blst_p2*()             { return &point; }
 };
 
-class P2s {
+class P2_Affines {
 private:
     struct p2_affine_no_init {
         blst_p2_affine point;
         p2_affine_no_init() { }
-        operator blst_p2_affine*() { return &point; }
+        operator blst_p2_affine*()              { return &point; }
+        operator const blst_p2_affine*() const  { return &point; }
     };
 
     std::vector<p2_affine_no_init> table;
     size_t wbits, npoints;
 
 public:
-    P2s() {}
 #ifndef SWIG
-    P2s(size_t wbits, const P2_Affine* points[], size_t npoints)
+    P2_Affines() {}
+    P2_Affines(size_t wbits, const P2_Affine* const points[], size_t npoints)
     {   this->wbits = wbits;
         this->npoints = npoints;
         table.resize(npoints << (wbits-1));
         blst_p2s_mult_wbits_precompute(table[0], wbits,
-                    reinterpret_cast<const blst_p2_affine **>(points), npoints);
+                        reinterpret_cast<const blst_p2_affine *const*>(points),
+                        npoints);
     }
-    P2s(size_t wbits, const P2_Affine points[], size_t npoints)
-    {   const P2_Affine* ptrs[2] = { points, nullptr };
-        P2s(wbits, ptrs, npoints);
+    P2_Affines(size_t wbits, const P2_Affine points[], size_t npoints)
+    {   const P2_Affine* const ptrs[2] = { points, nullptr };
+        P2_Affines(wbits, ptrs, npoints);
     }
-    P2s(size_t wbits, const std::vector<P2_Affine> points)
-    {   P2s(wbits, &points[0], points.size());   }
+    P2_Affines(size_t wbits, const std::vector<P2_Affine> points)
+    {   P2_Affines(wbits, &points[0], points.size());   }
 
-    P2s(size_t wbits, const P2* points[], size_t npoints)
+    P2_Affines(size_t wbits, const P2* const points[], size_t npoints)
     {   size_t cap = npoints << (wbits-1);
 
         this->wbits = wbits;
         this->npoints = npoints;
         table.resize(cap);
         blst_p2s_to_affine(table[cap-npoints],
-                           reinterpret_cast<const blst_p2 **>(points), npoints);
-        const blst_p2_affine* ptrs[2] = { table[cap-npoints], nullptr };
+                           reinterpret_cast<const blst_p2 *const*>(points),
+                           npoints);
+        const blst_p2_affine* const ptrs[2] = { table[cap-npoints], nullptr };
         blst_p2s_mult_wbits_precompute(table[0], wbits, ptrs, npoints);
     }
-    P2s(size_t wbits, const P2 points[], size_t npoints)
-    {   const P2* ptrs[2] = { points, nullptr };
-        P2s(wbits, ptrs, npoints);
+    P2_Affines(size_t wbits, const P2 points[], size_t npoints)
+    {   const P2* const ptrs[2] = { points, nullptr };
+        P2_Affines(wbits, ptrs, npoints);
     }
-    P2s(size_t wbits, const std::vector<P2> points)
-    {   P2s(wbits, &points[0], points.size());   }
+    P2_Affines(size_t wbits, const std::vector<P2> points)
+    {   P2_Affines(wbits, &points[0], points.size());   }
 
-    P2 mult_wbits(const byte* scalars[], size_t nbits)
+    P2_Affines(const P2* const points[], size_t npoints)
+    {   this->wbits = 0;
+        this->npoints = npoints;
+        table.resize(npoints);
+        blst_p2s_to_affine(table[0],
+                           reinterpret_cast<const blst_p2 *const*>(points),
+                           npoints);
+    }
+    P2_Affines(const P2 points[], size_t npoints)
+    {   const P2* const ptrs[2] = { points, nullptr };
+        P2_Affines(ptrs, npoints);
+    }
+    P2_Affines(const std::vector<P2> points)
+    {   P2_Affines(&points[0], points.size());   }
+
+    P2 mult(const byte* const scalars[], size_t nbits) const
     {   P2 ret;
-        void* scratch = malloc(blst_p2s_mult_wbits_scratch_sizeof(npoints));
-        blst_p2s_mult_wbits(ret, table[0], wbits, npoints,
-                                 scalars, nbits, scratch);
-        free(scratch);
+        limb_t* scratch;
+
+        if (wbits != 0) {
+            scratch = new limb_t[blst_p2s_mult_wbits_scratch_sizeof(npoints)/sizeof(limb_t)];
+            blst_p2s_mult_wbits(ret, table[0], wbits, npoints,
+                                     scalars, nbits, scratch);
+        } else {
+            scratch = new limb_t[blst_p2s_mult_pippenger_scratch_sizeof(npoints)/sizeof(limb_t)];
+            const blst_p2_affine* const ptrs[2] = { table[0], nullptr };
+            blst_p2s_mult_pippenger(ret, ptrs, npoints,
+                                         scalars, nbits, scratch);
+        }
+        delete[] scratch;
         return ret;
     }
 
-    static std::vector<P2_Affine> to_affine(const P2* points[], size_t npoints)
+    static std::vector<P2_Affine> from(const P2* const points[], size_t npoints)
     {   std::vector<P2_Affine> ret;
         ret.resize(npoints);
         blst_p2s_to_affine(ret[0],
-                           reinterpret_cast<const blst_p2 **>(points), npoints);
+                           reinterpret_cast<const blst_p2 *const*>(points),
+                           npoints);
         return ret;
     }
-    static std::vector<P2_Affine> to_affine(const P2 points[], size_t npoints)
-    {   const P2* ptrs[2] = { points, nullptr };
-        return to_affine(ptrs, npoints);
+    static std::vector<P2_Affine> from(const P2 points[], size_t npoints)
+    {   const P2* const ptrs[2] = { points, nullptr };
+        return from(ptrs, npoints);
     }
-    static std::vector<P2_Affine> to_affine(std::vector<P2> points)
-    {   return to_affine(&points[0], points.size());   }
+    static std::vector<P2_Affine> from(std::vector<P2> points)
+    {   return from(&points[0], points.size());   }
 #endif
 
-    static P2 mult_pippenger(const P2_Affine* points[], size_t npoints,
-                             const byte* scalars[], size_t nbits)
+    static P2 mult_pippenger(const P2_Affine* const points[], size_t npoints,
+                             const byte* const scalars[], size_t nbits)
     {   P2 ret;
-        void* scratch = malloc(blst_p2s_mult_pippenger_scratch_sizeof(npoints));
+        limb_t* scratch;
+        scratch = new limb_t[blst_p2s_mult_pippenger_scratch_sizeof(npoints)/sizeof(limb_t)];
         blst_p2s_mult_pippenger(ret,
-                    reinterpret_cast<const blst_p2_affine **>(points), npoints,
-                    scalars, nbits, scratch);
-        free(scratch);
+                    reinterpret_cast<const blst_p2_affine *const*>(points),
+                    npoints, scalars, nbits, scratch);
+        delete[] scratch;
         return ret;
     }
 #ifndef SWIG
     static P2 mult_pippenger(const P2_Affine points[], size_t npoints,
-                             const byte* scalars[], size_t nbits)
-    {   const P2_Affine *ptrs[2] = { points, nullptr };
+                             const byte* const scalars[], size_t nbits)
+    {   const P2_Affine* const ptrs[2] = { points, nullptr };
         return mult_pippenger(ptrs, npoints, scalars, nbits);
     }
     static P2 mult_pippenger(const std::vector<P2_Affine> points,
-                             const byte* scalars[], size_t nbits)
+                             const byte* const scalars[], size_t nbits)
     {   return mult_pippenger(&points[0], points.size(), scalars, nbits);   }
 #endif
 
-    static P2 add(const P2_Affine* points[], size_t npoints)
+    static P2 add(const P2_Affine* const points[], size_t npoints)
     {   P2 ret;
-        blst_p2s_add(ret, reinterpret_cast<const blst_p2_affine **>(points),
-                          npoints);
+        blst_p2s_add(ret,
+                     reinterpret_cast<const blst_p2_affine *const*>(points),
+                     npoints);
         return ret;
     }
 #ifndef SWIG
     static P2 add(const P2_Affine points[], size_t npoints)
-    {   const P2_Affine *ptrs[2] = { points, nullptr };
+    {   const P2_Affine* const ptrs[2] = { points, nullptr };
         return add(ptrs, npoints);
     }
     static P2 add(const std::vector<P2_Affine> points)
@@ -675,11 +822,16 @@ class PT {
 private:
     blst_fp12 value;
 
+    PT(const blst_fp12 *v)  { value = *v; }
 public:
     PT(const P1_Affine& p)  { blst_aggregated_in_g1(&value, p); }
-    PT(const P2_Affine& p)  { blst_aggregated_in_g2(&value, p); }
-    PT(const P2_Affine& p2, const P1_Affine& p1)
-    {   blst_miller_loop(&value, p2, p1);   }
+    PT(const P2_Affine& q)  { blst_aggregated_in_g2(&value, q); }
+    PT(const P2_Affine& q, const P1_Affine& p)
+    {   blst_miller_loop(&value, q, p);   }
+    PT(const P1_Affine& p, const P2_Affine& q) : PT(q, p) {}
+    PT(const P2& q, const P1& p)
+    {   blst_miller_loop(&value, P2_Affine(q), P1_Affine(p));   }
+    PT(const P1& p, const P2& q) : PT(q, p) {}
 
     PT dup() const          { return *this; }
     bool is_one() const     { return blst_fp12_is_one(&value); }
@@ -688,9 +840,13 @@ public:
     PT* sqr()               { blst_fp12_sqr(&value, &value);    return this; }
     PT* mul(const PT& p)    { blst_fp12_mul(&value, &value, p); return this; }
     PT* final_exp()         { blst_final_exp(&value, &value);   return this; }
+    bool in_group() const   { return blst_fp12_in_group(&value); }
+    void to_bendian(byte out[48*12]) const
+    {   blst_bendian_from_fp12(out, &value);   }
 
     static bool finalverify(const PT& gt1, const PT& gt2)
     {   return blst_fp12_finalverify(gt1, gt2);   }
+    static PT one() { return PT(blst_fp12_one()); }
 
 private:
     friend class Pairing;
@@ -793,6 +949,10 @@ public:
     {   return blst_pairing_merge(*this, *ctx);   }
     bool finalverify(const PT* sig = nullptr) const
     {   return blst_pairing_finalverify(*this, *sig);   }
+    void raw_aggregate(const P2_Affine* q, const P1_Affine* p)
+    {   blst_pairing_raw_aggregate(*this, *q, *p);   }
+    PT as_fp12()
+    {   return PT(blst_pairing_as_fp12(*this));   }
 };
 
 } // namespace blst
