@@ -12,6 +12,7 @@ LIDO_ADDRESSES = {
     Network.Rinkeby: "0xF4242f9d78DB7218Ad72Ee3aE14469DBDE8731eD",
     Network.Kintsugi: "0x3a6a994AC0CC96b6DDbaA99F10769384Fa14227B",
     Network.Kiln: "0x3E50180cf438e185ec52Ade55855718584541476",
+    Network.Zhejiang: "0xDe82ADEd58dA35add75Ea4676239Ca169c8dCD15",
 }
 
 NODE_OPS_ADDRESSES = {
@@ -21,6 +22,7 @@ NODE_OPS_ADDRESSES = {
     Network.Rinkeby: "0x776dFe7Ec5D74526Aa65898B7d77FCfdf15ffBe6",
     Network.Kintsugi: "0xeb7D01f713F59EFfB350D05b7AF66720373D4F41",
     Network.Kiln: "0xb849C5b35DC45277Bad5c6F2c6C77183d842367c",
+    Network.Zhejiang: "0xB099EC462e42Ac2570fB298B42083D7A499045D8",
 }
 
 
