@@ -10,6 +10,7 @@ class Network(IntEnum):
     Ropsten = 3
     Kintsugi = 1337702
     Kiln = 1337802
+    Zhejiang = 1337803
 
 
 # Network
@@ -20,6 +21,7 @@ GENESIS_FORK_VERSION = {
     Network.Rinkeby: bytes.fromhex("00000000"),
     Network.Kintsugi: bytes.fromhex("60000069"),
     Network.Kiln: bytes.fromhex("70000069"),
+    Network.Zhejiang: bytes.fromhex("00000069"),
 }
 
 # Existing withdrawal credentials on the chain
@@ -42,4 +44,5 @@ WITHDRAWAL_CREDENTIALS = {
     Network.Rinkeby: [],
     Network.Kintsugi: [],
     Network.Kiln: [],
+    Network.Zhejiang: [],
 }
